
require "java"

def getDataStore(fileURL) 
  Factory.openDataStore("gate.persist.SerialDataStore",fileURL)
end

def getCorpusFromDataStore(corpusName,fileURL)
  ds = getDataStore(fileURL)
  ids = ds.getLrIds("gate.corpora.SerialCorpusImpl")
  theid = ids.detect{|id| corpusName == ds.getLrName(id)}
  parms = Factory.newFeatureMap()
  parms.put(DataStore::DATASTORE_FEATURE_NAME,ds)
  parms.put(DataStore::LR_ID_FEATURE_NAME,theid)
  feats = Factory.newFeatureMap()
  Factory.createResource("gate.corpora.SerialCorpusImpl",parms,feats,"tmp")
end

# Copy documents from one corpus to another even when the corpora
# are from different datastores 
def corpusCopy(fromCorpusName,toCorpusName)
  fromCorpus = getCorpus(fromCorpusName)
  if fromCorpus == nil
    return "ERROR: corpus "+fromCorpusName.to_s+" not found"
  end
  toCorpus = getCorpus(toCorpusName)
  if toCorpus == nil
    return "ERROR: corpus "+toCorpusName.to_s+" not found"
  end
  toDatastore = toCorpus.getDataStore()
  if toDatastore == nil
    return "ERROR: toCorpus not stored in a datastore"
  end
  # Unfortunately, documents cannot be copied between different serial datastores
  # toCorpus.add(fromCorpus.get(i)) will not work if toCorpus and fromCorpus
  # are from different datastores.
  size = fromCorpus.size()
  print "Corpus size is ",size,"\n"
  for i in (0...size)
    d = fromCorpus.get(i);
    fromCorpus.unloadDocument(i, false);
    d.setDataStore(nil);
    d.setLRPersistenceId(nil);
    toDataStore.adopt(d,nil);
    toDataStore.sync(d);
    toCorpus.add(d);
    toCorpus.unloadDocument(d);
    Factory.deleteResource(d);
  end
  toDataStore.sync(toCorpus);  
end

# submit a query to a lucene searchable datastore within a corpus
def searchCorpus(corpusName,queryString)
  corpus = getCorpus(corpusName)
  if corpus == nil
    return "ERROR: corpus "+corpusName.to_s+" not found"
  end
  ds = corpus.getDataStore()
  if ds == nil
    return "ERROR: corpus not stored in a datastore"
  end
  searcher = ds.getSearcher()
  if searcher == nil
    return "ERROR: corpus not store in a searchable datastore"
  end
  parameters = HashMap.new()
  indexer = ds.getIndexer()
  iparms = indexer.getParameters()
  indexURL = iparms.get("INDEX_LOCATION_URL")
  indexFile = indexURL.getFile()
  newFile = java.io.File.new(indexFile)
  indexLocation = newFile.getAbsolutePath()
  indexLocations = java.utils.ArrayList.new()
  indexLocations.add(indexLocation)
  parameters.put("INDEX_LOCATIONS",indexLocations)
  parameters.put("CONTEXT_WINDOW",
end
