/*
 *  LRScriptConsole.java
 *
 *  Copyright (c) 2007-, 
 *    Austrian Research Institute for Aritificial Intelligence (OFAI)
 *
 *  Johann Petrak, 2007-06-23
 *
 *  $Id: LRScriptConsole.java  $
 *
 */

package at.ofai.gate.scripting;

import java.io.File;
import java.io.FileReader;
import java.net.URL;
import java.util.*;
import javax.script.*;
import java.io.*;

import gate.*;
import gate.creole.*;
import gate.persist.PersistenceException;


public class LRScriptConsole extends AbstractLanguageResource {

  protected URL       scriptUrl;
  public void setScriptUrl(URL scriptUrl) {
    this.scriptUrl = scriptUrl;
  }
  public URL getScriptUrl() {
    return scriptUrl;
  }
  
  protected String    scriptText;
  public void setScriptText(String scriptText) {
    this.scriptText = scriptText;
  }
  public String getScriptText() {
    return scriptText;
  }
  
  protected String    scriptLanguage;
  public void setScriptLanguage(String scriptLanguage) {
    this.scriptLanguage = scriptLanguage;
  }
  public String getScriptLanguage() {
    return scriptLanguage;
  }
  
  protected ScriptEngine scriptEngine;
  public void setScriptEngine(ScriptEngine engine) {
    this.scriptEngine = engine;
  }
  public ScriptEngine getScriptEngine() {
    return this.scriptEngine;
  }
  protected File initDir;
  protected String helpInfo = "";
  public String getHelpInfo() {
    return helpInfo;
  }
  @Override
  public Resource init() throws ResourceInstantiationException {
    throw new ResourceInstantiationException("Cannot use LRScriptConsole - use subclass!");
  }
  /** Initialise this resource, and return it. */
  protected void initEngine() throws ResourceInstantiationException {
    // This gets run after the constructor of the language-specific
    // subclass has completed and set scriptLanguage to the apropriate
    // value.
    try {
      
      // TODO: set properties before the engine is started? 
      // Factor out functions that are also needed by other classes
      // 
      scriptText = "";
      ScriptEngineManager scriptEngineManager  = new ScriptEngineManager(Gate.getClassLoader());
      // DEBUG: show available scripting engines
      // List<ScriptEngineFactory> fs = scriptEngineManager.getEngineFactories();
      // for(ScriptEngineFactory f : fs ) {
      //   System.out.println("Engine: "+f.getNames());
      // }
      // DEBUG END
      scriptEngine = scriptEngineManager.getEngineByName(scriptLanguage);
      if(scriptEngine == null) {
        throw new 
        ResourceInstantiationException("Could not init "+scriptLanguage+" engine!");
      }     
      // This uses the location of the LRScriptConsoleVR to figure out where
      // this plugin is located on disk. That information is then used to
      // find the directory that contains the init scripts.
      CreoleRegister reg = gate.Gate.getCreoleRegister();
      ResourceData myVR = reg.get("at.ofai.gate.scripting.LRScriptConsoleVR");
      if(myVR == null) {
        throw new ResourceInstantiationException(
            "Could not find the ResourceData for at.ofai.gate.scripting.LRScriptConsoleVR"
            );
      }
      URL myVRURL = myVR.getXmlFileUrl();
      java.io.File myVRFile = gate.util.Files.fileFromURL(myVRURL);
      initDir = new File(myVRFile.getParentFile(),"init");
      // read in and show the help information
      java.io.File helpFile = new File(myVRFile.getParentFile(),"doc/help_"+scriptLanguage+".txt");
      BufferedReader in = new BufferedReader(new FileReader(helpFile));
      String line = null;
      StringBuffer buffer = new StringBuffer();
      while ((line = in.readLine()) != null) {
        buffer.append(line+"\n");  
      }//end while
      in.close();
      helpInfo = buffer.toString();
      // load any properties for the plugin
      java.io.File propertiesFile = new File(myVRFile.getParentFile(),"run.properties");
      if(propertiesFile.exists()) {
        Properties pr = new Properties();
        FileInputStream inpr = new FileInputStream(propertiesFile);
        pr.load(inpr);
        in.close();
        // set System properties
        Set<Object> l = pr.keySet();
        for (Object o : l) {
          String k = (String)o;
          System.setProperty(k,pr.getProperty(k));
        }
      }
    } catch(Exception e) {
      throw new ResourceInstantiationException(e);
    }
  } // initEngine

  /**
   * Sets the parent LR of this LR.
   * Only relevant for LRs that support shadowing. Most do not by default.
   */
  @Override
  public void setParent(LanguageResource parentLR)
    throws PersistenceException,SecurityException {
    throw new UnsupportedOperationException();
  }

  /**
   * Returns the parent LR of this LR.
   * Only relevant for LRs that support shadowing. Most do not by default.
   */
  @Override
  public LanguageResource getParent()
    throws PersistenceException,SecurityException{
    throw new UnsupportedOperationException();
  }

  /**
   * Returns true of an LR has been modified since the last sync.
   * Always returns false for transient LRs.
   */
  public boolean isModified() {
    return false;
    // TODO: implement this?
  }

  /** Save: synchonise the in-memory image of the LR with the persistent
    * image.
    */
  public void sync() throws PersistenceException,SecurityException {
    throw new UnsupportedOperationException();
  }

  /** Sets the persistence id of this LR. To be used only in the
   *  Factory and DataStore code.
   */
  public void setLRPersistenceId(Object lrID){
    throw new UnsupportedOperationException();
  }

   /** Returns the persistence id of this LR, if it has been stored in
   *  a datastore. Null otherwise.
   */
  public Object getLRPersistenceId(){
    throw new UnsupportedOperationException();
  }

  /** Get the data store that this LR lives in. Null for transient LRs. */
  public DataStore getDataStore(){
    throw new UnsupportedOperationException();
  }

   /** Set the data store that this LR lives in. */
  public void setDataStore(DataStore dataStore) throws PersistenceException{
    throw new UnsupportedOperationException();
  }

  static public Resource getNamed(String name, String classname)
    throws  gate.util.GateException {
    if(classname == null || classname.equals("")) {
      classname = "gate.Resource";
    }
    Iterator i = Gate.getCreoleRegister().getAllInstances(classname).iterator();
    while(i.hasNext()) {
      Resource x = (Resource)i.next();
      if(x.getName().equals(name))
        return x;
    }
    return null;
  }

  static public Document getDocument(String name) throws  gate.util.GateException  {
    return (Document)getNamed(name,"gate.Document");
  }
  static public Corpus getCorpus(String name) throws  gate.util.GateException  {
    return (Corpus)getNamed(name,"gate.Corpus");
  }
  static public Controller getController(String name) throws  gate.util.GateException {
    return (Controller)getNamed(name,"gate.Controller");
  }

}