/*
 * PRScripting.java
 * 
 * Copyright (c) 2006, 2007, Austrian Research Institute for Artificial Intelligence 
 *
 * Johann Petrak, 2007-06-24
 *
 *  $Id: PRScripting.java  $
 *
 */
package at.ofai.gate.scripting;

import java.io.FileReader;
import java.net.URL;
import java.util.*;
import javax.script.*;
import java.io.*;

import gate.*;
import gate.creole.*;
import gate.event.*;
import gate.creole.gazetteer.*;
import gate.util.*;

/** Common superclass for using a script as a PR.  
 *  
 *  @author Johann Petrak
 */


// This does all the actual work. Scripting-language specific subclasses 
// only initialize things to the scripting-dependent values.
// Only init() is actually overriden in the subclasses and even that
// delegates most of the work to this class.
public class PRScripting
    extends gate.creole.AbstractLanguageAnalyser
    implements ControllerAwarePR {

  // bean getters/setters with their corresponding protected fields
  protected URL   scriptFileURL;
  public void setScriptFileURL(URL scriptFileURL) {
    this.scriptFileURL = scriptFileURL;
  }
  public URL getScriptFileURL() {
    return scriptFileURL;
  }
   
  private gate.Corpus prCorpus;
  
  public void controllerExecutionStarted(gate.Controller c) 
    throws ExecutionException {
    if(c instanceof SerialAnalyserController) {
      prCorpus = ((SerialAnalyserController)c).getCorpus();
    } else if(c instanceof ConditionalSerialAnalyserController) {
      prCorpus = ((ConditionalSerialAnalyserController)c).getCorpus();
    } else {
      throw new ExecutionException("This PR can only run in a SerialAnalyserController or ConditionalSerialAnalyserController");
    }
    try {
      scriptEngine.put("CORPUS", prCorpus);
      scriptEngine.eval(callStartupCorpus);
    } catch (Exception e) {
      System.err.println("Exception in startup: "+e);
      e.printStackTrace();
      // TODO: how to process what kind of exception here??
    }
  }
  
  public void controllerExecutionFinished(gate.Controller c) 
    throws ExecutionException {
    
    try {
      scriptEngine.put("CORPUS", prCorpus);
      scriptEngine.eval(callShutdownCorpus);
    } catch (Exception e) {
      System.err.println("Exception in shutdown: "+e);
      // TODO: how to process what kind of exception here??
    }
  }
  
  public void controllerExecutionAborted(gate.Controller c, java.lang.Throwable t)
    throws ExecutionException {
    // TODO
  }
  
  protected String scriptLanguage;
  protected ScriptEngine scriptEngine;
  protected String callStartupCorpus;
  protected String callShutdownCorpus;
  protected String callProcessDocument;
  // Initialize the scripting engine and load the script. A script should
  // define the following functions/methods to be useful as a PR
  //   startupCorpus(corpus)
  //   shutdownCorpus(corpus)
  //   processDocument(document)
  // In addition, any code on the global level will be executed when the
  // script is loaded, i.e. at the time of when this is run.
  protected void initEngine() throws ResourceInstantiationException {
    try {
      ScriptEngineManager scriptEngineManager  = new ScriptEngineManager();
      scriptEngine = scriptEngineManager.getEngineByName(scriptLanguage);
      if(scriptEngine == null) {
        throw new 
        ResourceInstantiationException("Could not init "+scriptLanguage+" engine!");
      }
      System.out.println("Evaluating script "+scriptFileURL+" with "+scriptLanguage);
      scriptEngine.eval(new FileReader(scriptFileURL.getFile()));
    } catch(Exception e) {
      throw new ResourceInstantiationException(e);
    }
  }

  
  
  /**
   * Run the processDocument() function/method of the script on a document
   */
    
  @Override
  public void execute() throws ExecutionException{
    if(document == null) {
      throw new ExecutionException(
        "No document to process!"
      );
    }
    try {
      scriptEngine.put("DOCUMENT",document);
      scriptEngine.eval(callProcessDocument);
    } catch (Exception e) {
      System.err.println("Exception in execute: "+e);
      // TODO: how to process what kind of exception here??
    }
  } // execute


} // DefaultGazetteer

